require 'rubyXL' # need install
require 'iso-639' # need install
require 'open-uri'

class LocGen
  TEMPORARY_SHEET_FILE = "temp_loc_sheet.xlsx"
  OUTPUT_FILE_NAME = "Localizable.strings"
  LOCAL_INDEX_KEY = "localIndex"
  SHEET_INDEX_KEY = "sheetIndex"
  LANG_KEY = "lang"

  def initialize(url, outputDir, comment, key, baselang)
    @url = url
    @outputDir = outputDir
    @commentColumnName = comment || "comment"
    @keyColumnName = key || "key"
    @baseLang = baselang
  end

  def process()
    downloadFile(@url, TEMPORARY_SHEET_FILE)
    if File.exists?(TEMPORARY_SHEET_FILE)
      parseXLSX(TEMPORARY_SHEET_FILE)
      prepareIndices()
      readLocalizations()
      writeFiles(@outputDir, OUTPUT_FILE_NAME)
      writeBaseLocalizable(@outputDir, OUTPUT_FILE_NAME)
      deleteTempFile(TEMPORARY_SHEET_FILE)
    end
  end

  private
  def downloadFile(url, tempFileName)
    File.open(tempFileName, "wb") do |saved_file|
      # the following "open" is provided by open-uri
      open(url, "rb") do |read_file|
        saved_file.write(read_file.read)
      end
    end
  end

  private
  def deleteTempFile(tempFileName)
    File.delete(tempFileName)
  end

  private
  def parseXLSX(file)
    @xlsx = RubyXL::Parser.parse(file)
    @sheet = @xlsx[0]
  end

  private
  def prepareIndices()
    @languages = []
    @localizations = []

    localIndex = 0
    firstRow = @sheet[0]

    firstRow && firstRow.cells.each_with_index { |cell,cellIndex|
      val = cell && cell.value
      # keyIndex column index
      if val == @keyColumnName
        @keyIndex = cellIndex
      elsif val == @commentColumnName
      # commentIndex column index
        @commentIndex = cellIndex
      else
        lang = val && ISO_639.find_by_code(val)
        if lang && lang.length > 0
          hash = {LANG_KEY => val, SHEET_INDEX_KEY => cellIndex, LOCAL_INDEX_KEY => localIndex}
          if !@languages.include?(hash)
            @languages.push(hash)
            @localizations[localIndex] = []
            localIndex = @localizations.length
          end
        end
      end
    }
  end

  private
  def readLocalizations()
    @sheet.each_with_index { |row,rowIndex|
      if rowIndex != 0
        if row
          # localized key
          if row.cells[@keyIndex]
            next if !row.cells[@keyIndex].value || row.cells[@keyIndex].value == ""
            key = row.cells[@keyIndex].value
          end
          # localized comment
          if row.cells[@commentIndex]
            comment = row.cells[@commentIndex].value
          end

          @languages.each_with_index{|hash,index|
            if row.cells[hash[SHEET_INDEX_KEY]]
              localization = row.cells[hash[SHEET_INDEX_KEY]].value
              unless key.to_s.strip.empty?
                @localizations[hash[LOCAL_INDEX_KEY]].push("//#{comment}\n\"#{key}\" = \"#{localization}\";")
              end
            end
          }
        end
      end
    }
  end

  private
  def writeFiles(dir, fileName)
    @languages.each_with_index{ |hash,index|
      localIndex = hash["localIndex"]
      langId = hash["lang"]
      dirName = "#{langId}.lproj"
      dirPath = File.join(dir, dirName)
      Dir.mkdir(dirPath) unless File.exists?(dirPath)
      File.write("#{dirPath}/#{fileName}", @localizations[localIndex].join("\n\n"))
    }
  end

  private
  def writeBaseLocalizable(dir, fileName)
    selectedHash = @languages[0]
    if @baseLang && @baseLang.length == 2
      hashes = @languages.select { |item|
        item[LANG_KEY] == @baseLang
      }
      if hashes.length > 0
        selectedHash = hashes[0]
      end
    end

    localIndex = selectedHash["localIndex"]
    dirName = "Base.lproj"
    dirPath = File.join(dir, dirName)
    Dir.mkdir(dirPath) unless File.exists?(dirPath)
    File.write("#{dirPath}/#{fileName}", @localizations[localIndex].join("\n\n"))
  end
end
